/***************************************************************************
 *
 * Filename     :  linuxentry.cpp
 * Content      :  Entry (and exit) points for the Linux MTXMEMMANAGER Driver
 *
 * COPYRIGHT (c) 2006  Matrox Electronic Systems Ltd.
 * All Rights Reserved
 *
 ****************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/uaccess.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/unistd.h>
#include <asm/bitops.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/interrupt.h>
#include <linux/version.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/ioctl.h>
#include <asm/msr.h>
#include <linux/init.h>
#include <linux/cdev.h>
#include <linux/list.h>
#include <linux/seq_file.h>

#if M_PUBLIC_RELEASE
#include "mtxmemmanager.h"
#else
#include "driver/mtxmemmanager.h"
#endif
#include "linux-interface.h"
#include "linuxentry.h"

#define MAX_DEVICE_OBJECT 225

#define B2MB(A) ((A)/1048576)

const char *MDrv_DriverName = "MTXMEMMANAGER";
MODULE_LICENSE("Proprietary");
MODULE_DESCRIPTION("Matrox Imaging Memory manager Module");
MODULE_AUTHOR("Matrox Electronic Systems Ltd.");

typedef struct _MEM_DRIVER_INSTANCE
{
   struct cdev       cdev;
   PMEM_DEVOBJECT    mem_device_object;
} MEM_DRIVER_INSTANCE;

static void   MEMLinuxAddCDevice(int devno, MEM_DRIVER_INSTANCE * inst);
static void   MEMLinuxDelCDevice( struct cdev * dev_handle);

static int mtxmem_open (struct inode *inode, struct file *filp);
static int mtxmem_close(struct inode *inode, struct file *filp);
static long mtxmem_unlocked_ioctl (struct file *filp, unsigned int cmd, unsigned long arg);
static int mtxmem_mmap(struct file *filp, struct vm_area_struct *vma);

static void mtxmem_init_proc(void);
static void mtxmem_remove_proc(void);
static ssize_t mtxmem_dbglevel_write_proc(struct file *file, const char __user *buffer, size_t count, loff_t *data);
static int mtxmem_dbglevel_show(struct seq_file *sfile, void *iterator);
static int mtxmem_dbglevel_open(struct inode *inode, struct file *file);
static int mtxmem_builddate_show(struct seq_file *sfile, void *iterator);
static int mtxmem_builddate_open(struct inode *inode, struct file *file);

static void *mtxmem_seq_start(struct seq_file *sfile, loff_t *index);
static void *mtxmem_seq_next(struct seq_file *sfile, void *iterator, loff_t *index);
static void mtxmem_seq_stop(struct seq_file *sfile, void *iterator);
static int mtxmem_seq_show(struct seq_file *s, void *iterator);
static int mtxmem_meminfo_open(struct inode *inode, struct file *file);
void initMmapLock (void);
void deleteMmapLock (void);

int MDrv_DbgLevel = DBG_CRIT|DBG_ERROR;
static dev_t   MEMLinuxDevId = 0;
static MEM_DRIVER_INSTANCE*  MEMGeneralDevice = NULL;
static char *registry_path = MEM_REGISTRY_PATH;
module_param (registry_path, charp, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC (registry_path, "Path to the registry");

#define DEVICE_NAME "mtxmemmanager"

static struct file_operations mtxmem_fops = {
   .owner   = THIS_MODULE,
   .unlocked_ioctl = mtxmem_unlocked_ioctl,
   .open    = mtxmem_open,
   .release = mtxmem_close,
   .mmap    = mtxmem_mmap,
};

static struct seq_operations mtxmem_seq_ops = {
   .start  = mtxmem_seq_start,
   .next   = mtxmem_seq_next,
   .stop   = mtxmem_seq_stop,
   .show   = mtxmem_seq_show
};

static struct file_operations mtxmem_meminfo_ops = {
   .owner   = THIS_MODULE,
   .open    = mtxmem_meminfo_open,
   .read    = seq_read,
   .llseek  = seq_lseek,
   .release = seq_release
};

static const struct file_operations mtxmem_builddate_ops = {
   .owner   = THIS_MODULE,
   .open    = mtxmem_builddate_open,
   .read    = seq_read,
   .llseek  = seq_lseek,
   .release = single_release
};

static const struct file_operations mtxmem_dbglevel_ops = {
   .owner   = THIS_MODULE,
   .open    = mtxmem_dbglevel_open,
   .read    = seq_read,
   .write   = mtxmem_dbglevel_write_proc,
   .llseek  = seq_lseek,
   .release = single_release
};

//---------------------------------------------------------------
//
// Name     : mtxmem_unlocked_ioctl
//
// Synopsys : execute an ioctl call from the user space
//
// Return   : 0 or error code
//
//---------------------------------------------------------------
static long mtxmem_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned long msg)
{
//   MEM_DRIVER_INSTANCE* memDrvInst = filp->private_data;
//   return MEMLinuxIoCtl(cmd, (void*)msg, memDrvInst->mem_device_object);
   return MEMLinuxIoCtl(cmd, (void*)msg, MEMGeneralDevice->mem_device_object, MEM_IOCTL_FROM_USER);
}

//---------------------------------------------------------------
//
// Name     : MtxMemKernelIoctl
//
// Synopsys : execute an ioctl call from kernel space
//
// Return   : 0 or error code
//
//---------------------------------------------------------------
int MtxMemKernelIoctl (int cmd, void *msg)
{
   return MEMLinuxIoCtl(cmd, (void*)msg, MEMGeneralDevice->mem_device_object, MEM_IOCTL_FROM_KERNEL);
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxReturnNoSys
//
// Synopsys : returns the ENOSYS code
//
// Return   : error code
//
//---------------------------------------------------------------
LINUX_DRV_STATUS MEMLinuxReturnNoSys(void)
{
   return ENOSYS;
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxReturnRestartSys
//
// Synopsys : returns the ERESTARTSYS code
//
// Return   : error code
//
//---------------------------------------------------------------
LINUX_DRV_STATUS MEMLinuxReturnRestartSys(void)
{
   return ERESTARTSYS;
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxReturnSuccess
//
// Synopsys : returns SUCCESS value
//
// Return   : returns 0
//
//---------------------------------------------------------------
LINUX_DRV_STATUS MEMLinuxReturnSuccess(void)
{
   return 0;
}

//---------------------------------------------------------------
//
// Name     : mtxmem_mmap
//
// Synopsys : map a physical address to a user space virtual address
//
// Return   : 0 or error code
//
//---------------------------------------------------------------
static int mtxmem_mmap(struct file *filp, struct vm_area_struct *vma)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
   unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;
   vma->vm_flags |= VM_RESERVED | VM_IO;
   if (io_remap_page_range(vma, vma->vm_start, offset, vma->vm_end - vma->vm_start, vma->vm_page_prot))
      return -EAGAIN;
#else
   vma->vm_flags |= VM_RESERVED | VM_IO | VM_LOCKED;
   if (remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff, vma->vm_end - vma->vm_start, vma->vm_page_prot))
      return -EAGAIN;
#endif
   return 0;
}

#ifdef TRACE_ACTIVE
//---------------------------------------------------------------
//
// Name     : MEMTrace
//
// Synopsys : prints the specified message in the kernel output
//
// Return   :
//
//---------------------------------------------------------------
void MEMTrace(long level, char *str, ...)
{
   if(level <= TRACE_LEVEL)
   {
      va_list ap;

      va_start(ap, str);
      vprintk(str, ap);

      va_end(ap);
   }
}
#endif

//---------------------------------------------------------------
//
// Name     : mtxmem_close
//
// Synopsys : close a connection to the driver
//
// Return   :
//
//---------------------------------------------------------------
static int mtxmem_close(struct inode *inode, struct file *filp)
{
   MEM_DRIVER_INSTANCE* memDrvInst = filp->private_data;
   if(MEMLinuxIoClose(memDrvInst->mem_device_object, (void*)filp) != MEMLinuxReturnSuccess())
      return -1;
   return 0;
}

//---------------------------------------------------------------
//
// Name     : mtxmem_open
//
// Synopsys : opens a connection to the driver
//
// Return   :
//
//---------------------------------------------------------------
static int mtxmem_open(struct inode *inode, struct file *filp)
{
   MEM_DRIVER_INSTANCE* memDrvInst = container_of(inode->i_cdev, MEM_DRIVER_INSTANCE, cdev);
   filp->private_data = memDrvInst;
   return MEMLinuxIoOpen(memDrvInst->mem_device_object, (void*)filp);
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxAddCDevice
//
// Synopsys : adds a cdev object for this driver
//
// Return   :
//
//---------------------------------------------------------------
void MEMLinuxAddCDevice(int devno, MEM_DRIVER_INSTANCE * inst)
{
	devno = MKDEV(MAJOR(MEMLinuxDevId),MINOR(MEMLinuxDevId) + devno);

	cdev_init(&inst->cdev, &mtxmem_fops);

	inst->cdev.owner = THIS_MODULE;
	inst->cdev.ops   = &mtxmem_fops;

	if (cdev_add(&inst->cdev, devno, 1) < 0)
	{
		DbgCritical("Failed to add cdev for mtxmemmanager\n");
	}
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxDelCDevice
//
// Synopsys : free a previously allocated CDEV
//
// Return   :
//
//---------------------------------------------------------------
void MEMLinuxDelCDevice( struct cdev * dev_handle)
{
	cdev_del( dev_handle );
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxCleanup
//
// Synopsys : - Unregister char device
//            - Remove Major Number
//
// OS       : Linux
//
//---------------------------------------------------------------
void MEMLinuxCleanup(void)
{
   mtxmem_remove_proc();
   MEMLinuxDelCDevice(&MEMGeneralDevice->cdev);
   MEMLinuxInstanceCleanup(MEMGeneralDevice->mem_device_object);

   MDrv_memfree(MEMGeneralDevice);
   MEMGeneralDevice = NULL;

   unregister_chrdev_region(MEMLinuxDevId, MAX_DEVICE_OBJECT);
   deleteMmapLock();
}

//---------------------------------------------------------------
//
// Name     : MEMLinuxInit
//
// Synopsys : - Allocate Major Number
//            - Register char device
//
// Return   : 0 or error code
//
//---------------------------------------------------------------
int MEMLinuxInit(void)
   {
   initMmapLock();
	if((alloc_chrdev_region(&MEMLinuxDevId, 0, MAX_DEVICE_OBJECT, DEVICE_NAME)) < 0)
	{
      DbgError ("Can't get a major number\n");
		return -ENODEV;
	}

	MEMGeneralDevice = MDrv_memalloc(sizeof(MEM_DRIVER_INSTANCE));
	if(MEMGeneralDevice == NULL)
	{
		DbgError ("not enougth memory\n");
		return -ENOMEM;
	}
	MDrv_memset(MEMGeneralDevice, 0, sizeof(MEM_DRIVER_INSTANCE));

   MEMLinuxCreateDefaultObject(&MEMGeneralDevice->mem_device_object);
	MEMLinuxAddCDevice(0, MEMGeneralDevice);

   mtxmem_init_proc();
   DbgTrace("Loaded\n");
	return 0;
}

#define ALLOC_ORDER (MAX_ORDER - 1)
#define NB_PAGE (1 << ALLOC_ORDER)
#define PAGE_BLOCK_SIZE (NB_PAGE * PAGE_SIZE)
const unsigned long MIN_FOR_OS = 8;
unsigned long linear_smallest_size(void)
{
   return PAGE_BLOCK_SIZE;
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : print_all_blocks
//
// Synopsis : Print, in kernel log, the starting and end address of every entry
//            in block_table.  Used only for debug.
//
// Arguments : struct page **block_table :
//             table_size :
//
////////////////////////////////////////////////////////////////////////////////
static void print_all_blocks (struct page **block_table, long table_size)
{
   long i = 0;
   printk ("===============================\n");
   for (i = 0; i < table_size; i++)
   {
      if (block_table[i])
         printk ("%ld %llX end at %llX\n", i, page_to_phys(block_table[i]), page_to_phys(block_table[i])+PAGE_BLOCK_SIZE);
   }
   printk ("===============================\n");
}


////////////////////////////////////////////////////////////////////////////////
//
// Name     : move_first
//
// Synopsis : Move used block at beginning of block_table
//
// Arguments : struct page **block_table
//             unsigned long table_size
//
////////////////////////////////////////////////////////////////////////////////
static void move_first (struct page **block_table, unsigned long table_size)
{
   long i = 0;
   long first_index = 1;
   long last_index = 0;
   long dest_index = 0;

   if (block_table[0])
      return;

   while ((first_index < table_size) && !block_table[first_index])
      first_index++;

   if (first_index == table_size)
      return;

   last_index = first_index;
   while (block_table[last_index])
      last_index++;

   for (i = first_index; i < last_index; i++)
   {
      block_table[dest_index] = block_table[i];
      dest_index++;
      block_table[i] = NULL;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : sort_blocks
//
// Synopsis : Sort block_table from the lowest address to the highest address
//
// Arguments : struct page **block_table
//             unsigned long table_size
//
////////////////////////////////////////////////////////////////////////////////
static void sort_blocks (struct page **block_table, unsigned long table_size)
{
   unsigned long i = 0;
   unsigned long j = 0;
   unsigned long biggest_index = 0;
   unsigned long last_index = table_size-1;
   struct page *temp = NULL;

   for (i = 0; i < last_index; i++)
   {
      biggest_index = 0;
      for (j = 0; j <= last_index-i; j++)
      {
         if ((page_to_phys(block_table[biggest_index]) < page_to_phys(block_table[j])))
            biggest_index = j;
      }
      temp = block_table[last_index-i];
      block_table[last_index-i] = block_table[biggest_index];
      block_table[biggest_index] = temp;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : free_page_range
//
// Synopsis : Free pages from block_table.
//
// Arguments : struct page **block_table
//             from : index of the first block to free
//             to   : index of the last+1 block to free
//
////////////////////////////////////////////////////////////////////////////////
static void free_page_range (struct page **block_table, long from, long to)
{
   long i = 0;
   long j = 0;
   long total_freed = 0;
   for (i = from; i < to; i++)
   {
      if (!block_table[i])
         continue;

      total_freed++;
      for (j = 0; j < NB_PAGE; j++)
         ClearPageReserved (block_table[i] + j);

      __free_pages (block_table[i], ALLOC_ORDER);
      block_table[i] = NULL;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : free_page_all
//
// Synopsis : Free all pages from block_table.
//
// Arguments : struct page **block_table
//
////////////////////////////////////////////////////////////////////////////////
static void free_page_all (struct page **block_table)
{
   unsigned long last_index = 0;

   while (block_table[last_index])
      last_index++;

   free_page_range(block_table, 0, last_index);
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : find_reserved_size
//
// Synopsis : Find the size in bytes of the reserved region in
//            block_table.  Assume continuous region to be at the
//            begining of block_table
//
// Arguments : struct page **block_table
//
// Return   : size in bytes of region contained in block_table
//
////////////////////////////////////////////////////////////////////////////////
static unsigned long find_reserved_size (struct page **block_table)
{
   unsigned long size = 0;
   long index = 0;
   while (block_table[index])
   {
      size += PAGE_BLOCK_SIZE;
      index++;
   }
   return size;
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : reserve_continuous
//
// Synopsis : Find the biggest (or one where size_wanted fit) continuous region
//            in block_table.
//
// Arguments : struct page **block_table
//             table_size : size of block_table
//             size_wanted : size in byte of the wanted region
//
// Return    : Physical address of the reserved block
//
////////////////////////////////////////////////////////////////////////////////
static unsigned long reserve_continuous (struct page **block_table, unsigned long table_size, unsigned long *size_wanted)
{
   unsigned long biggest_cont_index = 0;
   unsigned long biggest_cont_size = 0;
   unsigned long i = 0;
   unsigned long index = 0;
   unsigned long size = 1;
   unsigned long nb_block_wanted = (*size_wanted + PAGE_BLOCK_SIZE-1) / PAGE_BLOCK_SIZE;

   if (!block_table[0])
      {
      *size_wanted = 0;
      return 0;
      }

   sort_blocks(block_table, table_size);

   for (i = 0; i < table_size; i++)
   {
      if (!block_table[i])
         {
         index = i+1;
         continue;
         }

      if ((i == table_size-1) ||
          !block_table[i+1] ||
          ((page_to_phys(block_table[i])+PAGE_BLOCK_SIZE) != page_to_phys(block_table[i+1])))
      {
         MDrv_print("Possible region of size %lu MB\n", B2MB(size*PAGE_BLOCK_SIZE));
         if (size > biggest_cont_size)
         {
            biggest_cont_size = size;
            biggest_cont_index = index;
         }
         index = i+1;
         size = 1;
      }
      else
      {
         size++;
      }
   }

   if (biggest_cont_size > nb_block_wanted)
      biggest_cont_size = nb_block_wanted;

   /* Try to minimize the effects of an Out Of Memory condition so that the
      user can login and use milconfig to reduce the dma. */
   if ((table_size - biggest_cont_size) <= MIN_FOR_OS)
      {
      biggest_cont_size -= MIN_FOR_OS - (table_size - biggest_cont_size);
      if ((biggest_cont_size > 0) && (biggest_cont_size < table_size))
         biggest_cont_index += MIN_FOR_OS - (table_size - biggest_cont_size);
      else
         {
         biggest_cont_size = 0;
         biggest_cont_index = 0;
         }
      }

   *size_wanted = biggest_cont_size * PAGE_BLOCK_SIZE;
   if (biggest_cont_size == 0)
      {
      free_page_range(block_table, 0, table_size);
      schedule_timeout(500);
      return 0;
      }

   free_page_range(block_table, 0, biggest_cont_index);
   free_page_range(block_table, biggest_cont_index+biggest_cont_size, table_size);
   schedule_timeout(100);
   move_first(block_table, table_size);

   MDrv_print("Reserved a block of size %lu MB at %lX\n", B2MB(biggest_cont_size*PAGE_BLOCK_SIZE), page_to_phys(block_table[0]));
   return page_to_phys(block_table[0]);
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : linear_malloc
//
// Synopsis : Try to allocate a physically continuous region of size
//            size or the biggest possible.
//
// Arguments : size : Size in byte of the requested region
//             Below4GB : If memory should be under 4GB
//             BankData : To be passed to linear_free
//             allocated : Size really allocated
//
// Return   : physical address of the region allocated
//            or NULL
//
////////////////////////////////////////////////////////////////////////////////
unsigned long linear_malloc (unsigned long size, unsigned long Below4GB, void **BankData, unsigned long *allocated)
{
   long index = 0;
   long j = 0;
   unsigned long Flag = 0;
   unsigned long phys_addr = 0;
   unsigned long nb_max_blocks = ((size + PAGE_BLOCK_SIZE-1) / PAGE_BLOCK_SIZE) * 3 / 2 + MIN_FOR_OS;
   struct page *current_pages = NULL;
   struct page **available_block = NULL;
   struct page **block_table = MDrv_memalloc((nb_max_blocks+1)*sizeof(struct page *));
   memset (block_table, 0, (nb_max_blocks+1)*sizeof(struct page *));

   if (Below4GB)
      Flag = GFP_KERNEL | __GFP_DMA32 | __GFP_NOWARN;
   else
      Flag = GFP_KERNEL | __GFP_HIGHMEM | __GFP_NOWARN;

   BUG_ON(BankData == NULL);

   if(*BankData != NULL)
      return 0;

   DbgTrace("Trying to reserve %lu MB\n", B2MB(size));
   for (index = 0; index < nb_max_blocks; index++)
   {
      current_pages = alloc_pages(Flag, ALLOC_ORDER);
      if (NULL == current_pages)
      {
         DbgTrace("No more pages available\n");
         break;
      }

      for (j = 0; j < NB_PAGE; j++)
         SetPageReserved(current_pages + j);

      block_table[index] = current_pages;
   }

   *allocated = size;
   if ((phys_addr = reserve_continuous(block_table, index, allocated)))
   {
      int available_block_size = (*allocated + PAGE_BLOCK_SIZE-1) / PAGE_BLOCK_SIZE;
      available_block = MDrv_memalloc((available_block_size+1)*sizeof(struct page *)); // Last pointer must be null for linear_free to work
      MDrv_memcpy(available_block, block_table, (available_block_size+1)*sizeof(struct page *));
   }

   MDrv_memfree(block_table);
   *BankData = (void*)available_block;
   return phys_addr;
}

////////////////////////////////////////////////////////////////////////////////
//
// Name     : linear_free
//
// Synopsis : Free memory allocated by linear_malloc
//
// Arguments : physical_address : Address returned by linear_malloc
//
////////////////////////////////////////////////////////////////////////////////
void linear_free (void *BankData)
{
   struct page **available_block = (struct page **)BankData;
   BUG_ON(BankData == NULL);

   free_page_all(available_block);
   MDrv_memfree(available_block);
}

char *MtxGetRegistryPath (void)
{
   return registry_path;
}

static struct proc_dir_entry *MtxMemProc = NULL;
static struct proc_dir_entry *MtxProcEntry = NULL;
static void mtxmem_init_proc(void)
{
   struct proc_dir_entry *entry;

   MtxProcEntry = proc_mkdir("driver/mil", NULL);
   MtxMemProc = proc_mkdir("mtxmemmanager", MtxProcEntry);

   entry = proc_create_data("mtxmeminfo", 0444, MtxMemProc, &mtxmem_meminfo_ops, NULL);
   entry = proc_create_data("build_date", 0444, MtxMemProc, &mtxmem_builddate_ops, NULL);
   entry = proc_create_data("dbg_level", 0666, MtxMemProc, &mtxmem_dbglevel_ops, NULL);
}

static void mtxmem_remove_proc(void)
{
   remove_proc_entry("dbg_level", MtxMemProc);
   remove_proc_entry("build_date", MtxMemProc);
   remove_proc_entry("mtxmeminfo", MtxMemProc);
   remove_proc_entry("mtxmemmanager", MtxProcEntry);
   remove_proc_entry("driver/mil", NULL);
}

static int mtxmem_dbglevel_show(struct seq_file *sfile, void *iterator)
{
   seq_printf(sfile, "0x%x\n", MDrv_DbgLevel);
   return 0;
}

static int mtxmem_dbglevel_open(struct inode *inode, struct file *file)
{
   return single_open(file, mtxmem_dbglevel_show, NULL);
}

static ssize_t mtxmem_dbglevel_write_proc(struct file *file, const char __user *buffer, size_t count, loff_t *data)
{
   char *localBuf = NULL, *tmp = NULL;
   int newcount = count;
   int val = 0;
   int ret = 0;

   localBuf = MDrv_memalloc(count+1);
   if(localBuf == NULL)
      return -ENOMEM;

   tmp = localBuf;

   while((ret = copy_from_user(tmp, buffer, newcount)))
   {
      tmp += newcount - ret;
      buffer += newcount - ret;
      newcount -= newcount - ret;
   }

   localBuf[count] = '\0';

   sscanf(localBuf, "%x\n", &val);

   MDrv_DbgLevel = val;

   MDrv_memfree(localBuf);

   return count;
}

static int mtxmem_builddate_show(struct seq_file *sfile, void *iterator)
{
   seq_printf(sfile, "Module built on %s at %s\n",  __DATE__, __TIME__);
   return 0;
}

static int mtxmem_builddate_open(struct inode *inode, struct file *file)
{
   return single_open(file, mtxmem_builddate_show, NULL);
}

static void *mtxmem_seq_start(struct seq_file *sfile, loff_t *index)
{
   return MEMLinuxGetOSData(*index);
}

static void *mtxmem_seq_next(struct seq_file *sfile, void *iterator, loff_t *index)
{
   (*index)++;
   return MEMLinuxGetOSData(*index);
}

static void mtxmem_seq_stop(struct seq_file *sfile, void *iterator)
{
   return;
}

static int mtxmem_seq_show(struct seq_file *sfile, void *iterator)
{
   struct page **pages_block = (struct page**)iterator;
   seq_printf(sfile,
      "TotalDmaMemory:\t%lu bytes (%lu MB)\n"
      "StartAddress: 0x%llX\n",
      find_reserved_size(pages_block),
      B2MB(find_reserved_size(pages_block)),
      (unsigned long long)(page_to_phys(pages_block[0])));
   return 0;
}

static int mtxmem_meminfo_open(struct inode *inode, struct file *file)
{
   return seq_open(file, &mtxmem_seq_ops);
}



EXPORT_SYMBOL (MtxGetRegistryPath);
EXPORT_SYMBOL (MtxMemKernelIoctl);
EXPORT_SYMBOL (linear_malloc);
EXPORT_SYMBOL (linear_free);


EXPORT_SYMBOL (MEMAttach);
EXPORT_SYMBOL (MEMDetach);
EXPORT_SYMBOL (MEMAllocBuffer);
EXPORT_SYMBOL (MEMCreateBuffer);
EXPORT_SYMBOL (MEMFreeBuffer);
EXPORT_SYMBOL (MEMMapPtr);
EXPORT_SYMBOL (MEMUnmapPtr);
EXPORT_SYMBOL (MEMGetBufferInfo);
EXPORT_SYMBOL (MEMGetBufferInfoAtPosition);
EXPORT_SYMBOL (MEMBufferControl);
EXPORT_SYMBOL (MEMBufferInquire);
EXPORT_SYMBOL (MEMGetPhysicalAddress);
EXPORT_SYMBOL (MEMGetDriverInfo);
EXPORT_SYMBOL (MemConvertNameTagToValue);
EXPORT_SYMBOL (MemConvertValueToNameTag);


module_init(MEMLinuxInit);
module_exit(MEMLinuxCleanup);
