/*

Copyright (c) Matrox Electronic Systems. All rights reserved.

Module Name:

 mtxmemmanager_api.h

Abstract:

    This header will declare the C API (Application Programming Interface)
    for the mtxmemmanager (Matrox Memory manager) driver.

Revision History:

--*/

#ifndef __MTXMEMMANAGER_API_H__
#define __MTXMEMMANAGER_API_H__

// Generate environment define

#if defined(_WIN32)

#define MEM_API_USE_WINDOWS   1
#define MEM_API_USE_LINUX     0

#elif defined(__linux__)

#define MEM_API_USE_WINDOWS   0
#define MEM_API_USE_LINUX     1

#else

#error "Unknown Operating System"

#endif

#if defined(UNDER_RTSS)
#define MEM_API_USE_RT  1
#else
#define MEM_API_USE_RT  0
#endif

// now check for 32/64 bits

#if defined(_WIN64) || defined(__LP64__)

#define MEM_API_USE_64BIT     1

#else

#define MEM_API_USE_64BIT     0

#endif

// now check if we are in user or driver mode

#if (defined(DRIVER_CODE) || defined(_IM_DRIVER_CODE)) && !MEM_API_USE_RT   // this define will be present in the driver building script

#define MEM_API_DRIVER_CODE       1
#define MEM_API_USER_CODE         0

#else

#define MEM_API_DRIVER_CODE       0
#define MEM_API_USER_CODE         1

#endif


#if MEM_API_USE_RT && (!MEM_API_USER_CODE || MEM_API_DRIVER_CODE || !MEM_API_USE_64BIT || !MEM_API_USE_WINDOWS || MEM_API_USE_LINUX)
#error RTX not supported under this configuration
#endif
// type definition

typedef unsigned int       MEM_UINT32;
typedef unsigned long long MEM_UINT64;
typedef unsigned short     MEM_UINT16;
typedef unsigned char      MEM_UINT8;

#if MEM_API_USE_WINDOWS
typedef wchar_t            MEM_CHAR;
#else
typedef char               MEM_CHAR;
#endif

#if MEM_API_USE_64BIT
typedef MEM_UINT64       MEM_UINTPTR;
typedef MEM_UINT64       MEM_SIZE;
#else
typedef MEM_UINT32       MEM_UINTPTR;
typedef MEM_UINT32       MEM_SIZE;
#endif

typedef void*        MEM_HANDLE; // the MtxMemManager handle is an hidden structure
typedef MEM_HANDLE*  PMEM_HANDLE;

typedef MEM_UINT64   MEM_STATUS;


// for the names, we must keep the case identical
#if MEM_API_USER_CODE

#if MEM_API_USE_WINDOWS
#define MEM_KERNEL_NAME    
#define MEM_USER_NAME      "\\\\.\\mtxmemmanager"

#elif MEM_API_USE_LINUX
#define MEM_KERNEL_NAME    "/dev/mil/mtxmemmanager"
#define MEM_USER_NAME      MEM_KERNEL_NAME
#endif

#elif MEM_API_DRIVER_CODE

#if MEM_API_USE_WINDOWS

#define MEM_KERNEL_NAME    L"\\Device\\mtxmemmanager"
#define MEM_USER_NAME      L"\\DosDevices\\mtxmemmanager"

#elif MEM_API_USE_LINUX
#define MEM_KERNEL_NAME    "/dev/mil/mtxmemmanager"
#define MEM_USER_NAME      MEM_KERNEL_NAME
#endif
#endif

#ifdef __cplusplus
typedef bool MEM_BOOL;
#define MEM_NULL  0
#else
typedef int  MEM_BOOL;
#define true 1
#define false 0
#define MEM_NULL  ((void*)0)
#endif

// status code macros
#define MEM_STATUS_EXTRACT_ERROR(S)           (S & 0xffff0000)
#define MEM_STATUS_EXTRACT_DETAIL(S)          (S & 0x0000ffff)
#define MEM_STATUS_CREATE(S, D)            ((MEM_STATUS)((S & 0xffff0000) | (D & 0x0000ffff)))

// minimum and maximum macro
#ifndef MEM_MAX
#define MEM_MAX(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef MEM_MIN
#define MEM_MIN(a,b)            (((a) < (b)) ? (a) : (b))
#endif

// value definitions

// MAJOR status code

// when the upper 16 bit are 0, we have special code that will start with MEM_STATUS_
#define MEM_STATUS_SUCCESS                   0x00000000
#define MEM_STATUS_END_OF_LIST               0x00000001
#define MEM_STATUS_ALREADY_EXISTS            0x00000002
#define MEM_STATUS_NAME_NOT_FOUND            0x00000003
#define MEM_STATUS_BUFFER_TOO_SMALL          0x00000004

#define MEM_ERROR_INVALID_HANDLE             0x00010000
#define MEM_ERROR_INVALID_PARAMETER          0x00020000
#define MEM_ERROR_CANNOT_CREATE_HANDLE       0x00030000
#define MEM_ERROR_OS_ERROR                   0x00040000
#define MEM_ERROR_OUT_OF_MEMORY              0x00050000
#define MEM_ERROR_OUT_OF_HANDLE              0x00060000
#define MEM_ERROR_CANNOT_MAP                 0x00070000
#define MEM_ERROR_INVALID_CONTROL            0x00080000
#define MEM_ERROR_HEAP                       0x00090000

// MINOR status code (code 0 is reserved for MEM_STATUS_SUCCESS)
#define MEM_DETAIL_OUT_OF_MEMORY             0x00000001
#define MEM_DETAIL_DRIVER_HANDLE             0x00000002
#define MEM_DETAIL_NULL_BUFFER_POINTER       0x00000003
#define MEM_DETAIL_NULL_VIRTUAL_POINTER      0x00000004
#define MEM_DETAIL_IO_CONTROL                0x00000005
#define MEM_DETAIL_CANNOT_OPEN_DRIVER        0x00000006
#define MEM_DETAIL_BUFFER_HANDLE             0x00000007
#define MEM_DETAIL_NULL_BUFFER_INFO          0x00000008
#define MEM_DETAIL_NULL_PHYSICAL_POINTER     0x00000009
#define MEM_DETAIL_NULL_DRIVER_INFO          0x0000000A
#define MEM_DETAIL_MEMORY_BANK               0x0000000B
#define MEM_DETAIL_BUFFER_MEMORY             0x0000000C
#define MEM_DETAIL_BUFFER_STRUCT_MEMORY      0x0000000D
#define MEM_DETAIL_INTERNAL_MEMORY           0x0000000E
#define MEM_DETAIL_INTERNAL_ERROR            0x0000000F
#define MEM_DETAIL_KERNEL_MAP_FUNCTION       0x00000010
#define MEM_DETAIL_CANNOT_OPEN_SECTION       0x00000011
#define MEM_DETAIL_CANNOT_REF_OBJECT         0x00000012
#define MEM_DETAIL_CANNOT_MAP_VIEW           0x00000013
#define MEM_DETAIL_INEXISTANT                0x00000014
#define MEM_DETAIL_UNIMPLEMENTED             0x00000015
#define MEM_DETAIL_FLAG                      0x00000016
#define MEM_DETAIL_WRONG_PROCESS             0x00000017
#define MEM_DETAIL_BUFFER_MEMORY_LOW4GB      0x00000018
#define MEM_DETAIL_BUFFER_MEMORY_HIGH4GB     0x00000019
#define MEM_DETAIL_HIGH_LOW_4GB_SET_TOGETHER 0x0000001A
#define MEM_DETAIL_INCORRECT_CACHE_MODE      0x0000001B
#define MEM_DETAIL_BUFFER_MEMORY_CACHE       0x0000001C
#define MEM_DETAIL_BUFFER_MEMORY_NOCACHE     0x0000001D
#define MEM_DETAIL_BUFFER_MEMORY_WC          0x0000001E
#define MEM_DETAIL_NULL_VALUE_POINTER        0x0000001F
#define MEM_DETAIL_INVALID_CONTROL_ITEM      0x00000020
#define MEM_DETAIL_INVALID_MODE              0x00000021
#define MEM_DETAIL_LIMIT_ATTAINED            0x00000022
#define MEM_DETAIL_INVALID_HEAP              0x00000023
#define MEM_DETAIL_ALREADY_ALLOCATED         0x00000024
#define MEM_DETAIL_BUFFER_MEMORY_TOO_BIG     0x00000025
#define MEM_DETAIL_BAD_IRQL                  0x00000026
#define MEM_DETAIL_NAME_TOO_LONG             0x00000027
#define MEM_DETAIL_NULL_NAME                 0x00000028
#define MEM_DETAIL_EMPTY_NAME                0x00000029
#define MEM_DETAIL_MULTIPLE_NAMES            0x0000002A
#define MEM_DETAIL_INDEX_OUT_OF_RANGE        0x0000002B

// predefined parameters
#define MEM_DEFAULT_BANK               ((MEM_UINT32)-1)
#define MEM_START_POSITION             0
#define MEM_MAX_NUM_OF_BANK            256
#define MEM_MAX_NAME_LENGTH            128
#define MEM_MAX_NAME_BYTE_LENGTH       (MEM_MAX_NAME_LENGTH * sizeof(MEM_CHAR))
#define MEM_CREATE_BANK                (MEM_MAX_NUM_OF_BANK + 1)
#define MEM_DEMAND_BANK                (MEM_MAX_NUM_OF_BANK + 2)
#define MEM_USER_BANK_OFFSET           512
#define MEM_USER_BANK(N)               (MEM_USER_BANK_OFFSET + N)
#define MEM_MAX_NUM_OF_USER_BANK       16
#define MEM_IS_USER_BANK(N)            ((N >= MEM_USER_BANK_OFFSET) && (N < (MEM_USER_BANK(MEM_MAX_NUM_OF_USER_BANK))))

#define MEM_BANK_FLAG_NON_PAGED        0x1
#define MEM_BANK_FLAG_MAXMEM           0x2
#define MEM_BANK_FLAG_USER             0x4
#define MEM_BANK_FLAG_DELETING         0x8

// special mode flag value
#define MEM_MODE_HYBRID_4GB            1

// control codes
#if MEM_API_USE_LINUX
#include <asm/ioctl.h>
typedef struct _MtxMemMsgPack
{
   void*       send;      // data send to kernel
   void*       receive;   // data returned from kernel
   MEM_UINT32  inSize;    // data size of send msg
   MEM_UINT32  outSize;   // data size of returned msg
   MEM_UINT32  iocode;
   MEM_UINT32  padding;
}MtxMemMsgPack;

#define MEM_LINUX_CONTROL_ID           0xF1
#define MEM_LINUX_CONTROL              _IOWR(MEM_LINUX_CONTROL_ID, 0, struct _MtxMemMsgPack)
#endif

#define MEM_MAGIC_NUMBER               0
#define MEM_MAKE_VALID_IOCTL(A)        (MEM_MAGIC_NUMBER | A)

#define MEM_CONTROL_ALLOC_BUFFER       MEM_MAKE_VALID_IOCTL(0)
#define MEM_CONTROL_CREATE_BUFFER      MEM_MAKE_VALID_IOCTL(1)
#define MEM_CONTROL_FREE_BUFFER        MEM_MAKE_VALID_IOCTL(2)
#define MEM_CONTROL_MAPPING_BUFFER     MEM_MAKE_VALID_IOCTL(3)
#define MEM_CONTROL_INFO_BUFFER        MEM_MAKE_VALID_IOCTL(4)
#define MEM_CONTROL_BUFFER_CONTROL     MEM_MAKE_VALID_IOCTL(5)
#define MEM_CONTROL_GET_PHYSICAL       MEM_MAKE_VALID_IOCTL(6)
#define MEM_CONTROL_GET_DRIVER_INFO    MEM_MAKE_VALID_IOCTL(7)
#define MEM_CONTROL_BUFFER_INQUIRE     MEM_MAKE_VALID_IOCTL(8)
#define MEM_CONTROL_HEAP               MEM_MAKE_VALID_IOCTL(9)
#define MEM_CONTROL_ALLOC_NAMED_BUF    MEM_MAKE_VALID_IOCTL(10)
#define MEM_CONTROL_OPEN_NAMED_BUF     MEM_MAKE_VALID_IOCTL(11)
#define MEM_CONTROL_BUF_REG_NAME       MEM_MAKE_VALID_IOCTL(12)
#define MEM_CONTROL_FETCH_NAME_ID      MEM_MAKE_VALID_IOCTL(13)
#define MEM_CONTROL_BUFFER_DATA        MEM_MAKE_VALID_IOCTL(14)
#define MEM_NUM_OF_CONTROL             15 // must be one more than the last control

// version information
#define MEM_DRIVER_INFO_VERSION1       1
#define MEM_DRIVER_INFO_VERSION2       2
#define MEM_DRIVER_INFO_VERSION3       3
#define MEM_DRIVER_INFO_VERSION4       4
#define MEM_DRIVER_INFO_VERSION5       5
#define MEM_DRIVER_INFO_VERSION6       6
#define MEM_DRIVER_INFO_VERSION7       7
#define MEM_DRIVER_INFO_VERSION8       8

#define MEM_DRIVER_INFO_VERSION        MEM_DRIVER_INFO_VERSION8

#define MEM_BUFFER_INFO_VERSION1       1
#define MEM_BUFFER_INFO_VERSION2       2

#define MEM_BUFFER_INFO_VERSION        MEM_BUFFER_INFO_VERSION2

#define MEM_DRIVER_CREATE_VERSION(MAJOR,MINOR)  ((MEM_UINT32)(((MAJOR & 0xFFFF) << 16) + (MINOR & 0xFFFF)))

#define MEM_DRIVER_MINOR_VERSION       0
#define MEM_DRIVER_MAJOR_VERSION       12  // increment the major version only with an API modification
#define MEM_DRIVER_VERSION             MEM_DRIVER_CREATE_VERSION(MEM_DRIVER_MAJOR_VERSION, MEM_DRIVER_MINOR_VERSION)

#define MEM_DRIVER_GET_MAJOR_VERSION(V)   ((MEM_UINT32)(((V) >> 16) & 0xffff))
#define MEM_DRIVER_GET_MINOR_VERSION(V)   ((MEM_UINT32)((V) & 0xffff))

// user flags
#define MEM_LOW_4GB                    0x000001
#define MEM_HIGH_4GB                   0x000002
#define MEM_ALIGNMENT                  0x000004
#define MEM_ALIGN(N)                   MEM_ALIGNMENT | ( (N & 0x3f) << 3)
#define MEM_ALIGN_MASK                 ((MEM_UINT64)(0x3F << 3))
#define MEM_CACHE                      0x000200
#define MEM_NO_CACHE                   0x000400
#define MEM_WRITE_COMBINING            0x000800
#define MEM_FRONTIER_FLAG              0x001000
#define MEM_FRONTIER(N)                MEM_FRONTIER_FLAG | (( N & 0x3F) << 13)
#define MEM_FRONTIER_MASK              ((MEM_UINT64)(0x3F << 13))
#define MEM_FRONTIER_MAX_VALUE         64
#define MEM_DELETING                   0x080000
#define MEM_NEXT_FLAG                  0x100000

#define MEM_GET_ALIGNMENT(N)           ((N & MEM_ALIGNMENT) ? ((MEM_UINT64)1 << ((N >> 3) & 0x3f)) : 0)
#define MEM_GET_FRONTIER(N)            ((N & MEM_FRONTIER_FLAG) ? (MEM_UINT64)((MEM_UINT64)1 << ((N >> 13) & 0x3f)) : (((MEM_UINT64)1) << 0x3F))
#define MEM_GET_ALIGNMENT_EXP(N)       ((N & MEM_ALIGNMENT) ? (((N >> 3) & 0x3f)) : 0)
#define MEM_GET_FRONTIER_EXP(N)        ((N & MEM_FRONTIER_FLAG) ? (((N >> 13) & 0x3f)) : 0x3F)
#define MEM_ALLOC_IN_LOW_4GB(N)        (N & MEM_LOW_4GB)
#define MEM_ALLOC_IN_HIGH_4GB(N)       (N & MEM_HIGH_4GB)

// control flags
#define MEM_CTRL_BUFFER_MAP            1

// MEM_CTRL_BUFFER_MAP item values
#define MEM_KERNEL_MAP                 1
#define MEM_KERNEL_UNMAP               2

// inquire flag
#define MEM_INQ_KERNEL_VIRTUAL_PTR     1

// internal flags
#define MEM_INTERNAL_FLAG_MAP_BUFFER   0x1
#define MEM_INTERNAL_FLAG_FROM_KERNEL  0x2

#define MTXMEMMANAGER_SAFE_BOOT        1
#define MTXMEMMANAGER_FORCE_ONDEMAND   2
#define MTXMEMMANAGER_DISABLE_ONDEMAND 4
#define MTXMEMMANAGER_ENABLE_HYBRID    8
#define MTXMEMMANAGER_PREMAP_ALL       0x10

// for legacy status report
#define MTXMEMMANAGER_UNKNOWN_MODE           1
#define MTXMEMMANAGER_FOUND_PREV_BLOCK       2
#define MTXMEMMANAGER_END_IN_NP              4
#define MTXMEMMANAGER_START_IN_NP            8
#define MTXMEMMANAGER_NP_IN_RAM              0x10
#define MTXMEMMANAGER_OUT_MEMORY_MEMRANGE    0x20
#define MTXMEMMANAGER_NOT_FOUND_PREV_BLOCK   0x40

// mapping operation mode
#define MEM_MAPPING_MODE_MAP           0x1
#define MEM_MAPPING_MODE_UNMAP         0x2
#define MEM_MAPPING_FROM_KERNEL        0x4

// heap operation mode
#define MEM_HEAP_CREATE_USER           0x1
#define MEM_HEAP_FREE_USER             0x2

// register name flags
#define MEM_REG_NAME_CAN_EXIST         0x1

// buffer data flags
#define MEM_DATA_STORE                 0x1
#define MEM_DATA_FETCH                 0x2

// index for buffer data
#define MEM_DATA_BUFFER_INFO           0
#define MEM_DATA_BYTE_OFFSET           1

// various definition
#define FILE_DEVICE_MEMMNGR            0xba45
#define MEM_IOCTL_INDEX                2048

#define MTXMEMMANAGER_MEM_INVALID         0x0
#define MTXMEMMANAGER_MEM_PCI_CHUNK       0x1
#define MTXMEMMANAGER_MEM_NON_PAGED       0x2
#define MTXMEMMANAGER_MEM_ON_DEMAND       0x4
#define MTXMEMMANAGER_MEM_NP_THRESHOLD    0x8
#define MTXMEMMANAGER_MEM_DEFAULT         0x10

#if MEM_API_USE_WINDOWS

#define MEM_IOCTL_CODE(X) CTL_CODE(FILE_DEVICE_MEMMNGR, MEM_IOCTL_INDEX + X, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define EXTRACT_IOCTL_MEMMNGR_CODE(x)  ((((MEM_UINT32)(x) >> 2) - MEM_IOCTL_INDEX) & 0xfff)

#endif
// structure definition

typedef struct _MEM_DRIVER_INFORMATION_VER1
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT64 Reserved1;
   MEM_UINT64 Reserved2;
} MEM_DRIVER_INFORMATION_VER1;


typedef struct _MEM_DRIVER_INFORMATION_VER2
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 NonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
} MEM_DRIVER_INFORMATION_VER2;

typedef struct _MEM_DRIVER_INFORMATION_VER3
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 Reserved;
} MEM_DRIVER_INFORMATION_VER3;

typedef struct _MEM_DRIVER_INFORMATION_VER4
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 NumOfUserBank;
   MEM_UINT64 UserBankBufHandle;
} MEM_DRIVER_INFORMATION_VER4;

typedef struct _MEM_DRIVER_INFORMATION_VER5
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 NumOfUserBank;
   MEM_UINT64 UserBankBufHandle;
   MEM_UINT64 BiggestFreeBlock;
} MEM_DRIVER_INFORMATION_VER5;

typedef struct _MEM_DRIVER_INFORMATION_VER6
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 NumOfUserBank;
   MEM_UINT64 UserBankBufHandle;
   MEM_UINT64 BiggestFreeBlock;
   MEM_UINT32 SpecialMode;
   MEM_UINT32 MinChunkSize;
   MEM_UINT32 Lower4GBMaxSize;
   MEM_UINT32 Lower4GBMinChunk;
} MEM_DRIVER_INFORMATION_VER6;

typedef struct _MEM_DRIVER_INFORMATION_VER7
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 NumOfUserBank;
   MEM_UINT64 UserBankBufHandle;
   MEM_UINT64 BiggestFreeBlock;
   MEM_UINT32 SpecialMode;
   MEM_UINT32 MinChunkSize;
   MEM_UINT32 Lower4GBMaxSize;
   MEM_UINT32 Lower4GBMinChunk;
   MEM_UINT32 LargestBlock;
   MEM_UINT32 SmallestBlock;
} MEM_DRIVER_INFORMATION_VER7;

typedef struct _MEM_DRIVER_INFORMATION_VER8
   {
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BaseAddress;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 FreeMemInByte;
   MEM_UINT64 UsedMemInByte;
   MEM_UINT64 Version;
   MEM_UINT64 NumOfBank;
   MEM_UINT64 BankFlags;
   MEM_UINT64 NonPagedFlags;
   MEM_UINT64 KernelAddress;
   MEM_UINT32 NonPagedMode;
   MEM_UINT32 StartupNonPagedSize;
   MEM_UINT32 ChunkSize;
   MEM_UINT32 OSMemory;
   MEM_UINT32 BootFlags;
   MEM_UINT32 Threshold;
   MEM_UINT32 RegNonPagedMode;
   MEM_UINT32 RegChunkSize;
   MEM_UINT32 StartupNonPagedMode;
   MEM_UINT32 StartupChunkSize;
   MEM_UINT32 RegNonPagedSize;
   MEM_UINT32 NumOfUserBank;
   MEM_UINT64 UserBankBufHandle;
   MEM_UINT64 BiggestFreeBlock;
   MEM_UINT32 SpecialMode;
   MEM_UINT32 MinChunkSize;
   MEM_UINT32 Lower4GBMaxSize;
   MEM_UINT32 Lower4GBMinChunk;
   MEM_UINT32 LargestBlock;
   MEM_UINT32 SmallestBlock;
   MEM_UINT32 PhysicalRamInstalled;
   MEM_UINT32 MaxNonPaged;
   MEM_UINT32 StatusReport;
   MEM_UINT32 Reserved1;
   MEM_UINT64 Reserved2;
   } MEM_DRIVER_INFORMATION_VER8;


typedef MEM_DRIVER_INFORMATION_VER8 MEM_DRIVER_INFORMATION;

typedef struct _MEM_BUFFER_INFO_VER1
{
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BufferHandle;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 PhysAddr;
   MEM_UINT64 OwnerTag;
   MEM_UINT64 Flags;
   MEM_UINT64 ProcessId;
   MEM_UINT64 NameTag;
   MEM_UINT64 BankNumber;
   MEM_UINT64 Reserved1;
   MEM_UINT64 Reserved2;
   MEM_UINT64 Reserved3;
} MEM_BUFFER_INFO_VER1;

typedef struct _MEM_BUFFER_INFO_VER2
   {
   MEM_UINT32 StructVersion;
   MEM_UINT32 StructSize;
   MEM_UINT64 BufferHandle;
   MEM_UINT64 SizeInByte;
   MEM_UINT64 PhysAddr;
   MEM_UINT64 OwnerTag;
   MEM_UINT64 Flags;
   MEM_UINT64 ProcessId;
   MEM_UINT64 NameTag;
   MEM_UINT64 BankNumber;
   MEM_UINT64 NameLength;
   MEM_CHAR   Name[MEM_MAX_NAME_LENGTH];
   MEM_UINT64 Reserved1;
   MEM_UINT64 Reserved2;
   MEM_UINT64 Reserved3;
   } MEM_BUFFER_INFO_VER2;


typedef MEM_BUFFER_INFO_VER2  MEM_BUFFER_INFO;

// structure definition for the user/kernel interface

typedef struct _MEM_IOCTL_ALLOC_BUFFER
{
   MEM_UINT64  BufferSize;
   MEM_UINT64  MemoryBank;
   MEM_UINT64  Flags;
   MEM_UINT64  ProcessId;
   MEM_UINT64  Tag;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  VirtualAddress;
   MEM_UINT64  PhysAddress;
   MEM_UINT64  NameTag;
   MEM_UINT64  InternalFlags;
   MEM_STATUS  IOStatus;
} MEM_IOCTL_ALLOC_BUFFER;

typedef struct _MEM_IOCTL_ALLOC_NAMED_BUFFER
   {
   MEM_UINT64  BufferSize;
   MEM_UINT64  MemoryBank;
   MEM_UINT64  Flags;
   MEM_UINT64  ProcessId;
   MEM_UINT64  Tag;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  VirtualAddress;
   MEM_UINT64  PhysAddress;
   MEM_UINT64  NameTag;
   MEM_UINT64  InternalFlags;
   MEM_UINT64  NameLength;
   MEM_CHAR    Name[MEM_MAX_NAME_LENGTH];
   MEM_STATUS  IOStatus;
   } MEM_IOCTL_ALLOC_NAMED_BUFFER;


typedef struct _MEM_IOCTL_CREATE_BUFFER
{
   MEM_UINT64  PhysAddress;
   MEM_UINT64  BufferSize;
   MEM_UINT64  ProcessId;
   MEM_UINT64  Tag;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  VirtualAddress;
   MEM_UINT64  InternalFlags;
   MEM_UINT64  NameTag;
   MEM_STATUS  IOStatus;
   MEM_UINT64  Flags;
} MEM_IOCTL_CREATE_BUFFER;

typedef struct _MEM_IOCTL_FREE_BUFFER
{
   MEM_UINT64  BufferHandle;
   MEM_UINT64  ProcessId;
   MEM_UINT64  Tag;
   MEM_STATUS  IOStatus;
} MEM_IOCTL_FREE_BUFFER;

typedef struct _MEM_IOCTL_MAPPING
{
   MEM_UINT64  BufferHandle;
   MEM_UINT64  VirtPtr;
   MEM_UINT64  Operation;
   MEM_STATUS  IOStatus;
   MEM_UINT64  PhysAddress;
   MEM_UINT64  BufferSize;
} MEM_IOCTL_MAPPING;

typedef struct _MEM_IOCTL_INFO_BUFFER
{
   MEM_UINT64  BufferHandle;
   MEM_UINT64  Position;
   MEM_UINT64  DoGetAtPosition;
   MEM_UINT64  Version;
   MEM_STATUS  IOStatus;
   union _MEM_VERSIONS_INFO_BUFFER
   {
      MEM_BUFFER_INFO_VER1   Version1;
      MEM_BUFFER_INFO_VER2   Version2;
   } versions;
} MEM_IOCTL_INFO_BUFFER;

typedef struct _MEM_IOCTL_CONTROL_BUFFER
{
   MEM_UINT64  BufferHandle;
   MEM_UINT64  ControlType;
   MEM_UINT64  ControlValue;
   MEM_STATUS  IOStatus;
} MEM_IOCTL_CONTROL_BUFFER;

typedef struct _MEM_IOCTL_INQUIRE_BUFFER
{
   MEM_STATUS  IOStatus;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  InquireType;
   MEM_UINT64  InquireValue;
} MEM_IOCTL_INQUIRE_BUFFER;

typedef struct _MEM_IOCTL_GET_PHYSICAL
{
   MEM_UINT64  VirtAddress;
   MEM_UINT64  PhysAddress;
   MEM_STATUS  IOStatus;
} MEM_IOCTL_GET_PHYSICAL;

typedef struct _MEM_IOCTL_HEAP
{
   MEM_UINT64  BufHandle;
   MEM_UINT64  HeapHandle;
   MEM_UINT64  Mode;
   MEM_STATUS  IOStatus;
} MEM_IOCTL_HEAP;

typedef struct _MEM_IOCTL_GET_DRIVER_INFO
{
   MEM_UINT64  MemoryBank;
   MEM_UINT64  Version;
   MEM_STATUS  IOStatus;
   union _MEM_VERSIONS_DRV_INFO
   {
      MEM_DRIVER_INFORMATION_VER1   Version1;
      MEM_DRIVER_INFORMATION_VER2   Version2;
      MEM_DRIVER_INFORMATION_VER3   Version3;
      MEM_DRIVER_INFORMATION_VER4   Version4;
      MEM_DRIVER_INFORMATION_VER5   Version5;
      MEM_DRIVER_INFORMATION_VER6   Version6;
      MEM_DRIVER_INFORMATION_VER7   Version7;
      MEM_DRIVER_INFORMATION_VER8   Version8;
   } versions;
} MEM_IOCTL_GET_DRIVER_INFO;

typedef struct _MEM_IOCTL_OPEN_NAMED_BUFFER
   {
   MEM_CHAR    Name[MEM_MAX_NAME_LENGTH];
   MEM_UINT64  Flags;
   MEM_UINT64  InternalFlags;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  VirtualAddress;
   MEM_UINT64  PhysAddress;
   MEM_UINT64  BufferSize;
   MEM_STATUS  IOStatus;
   } MEM_IOCTL_OPEN_NAMED_BUFFER;

typedef struct _MEM_IOCTL_BUFFER_REGISTER_NAME
   {
   MEM_UINT64  BufferHandle;
   MEM_UINT64  Flags;
   MEM_CHAR    Name[MEM_MAX_NAME_LENGTH];
   MEM_UINT64  NameLength;
   MEM_STATUS  IOStatus;

   } MEM_IOCTL_BUFFER_REGISTER_NAME;

typedef struct _MEM_IOCTL_FETCH_NAME_ID
   {
   MEM_UINT64  BufferHandle;
   MEM_UINT64  Flags;
   MEM_CHAR    Name[MEM_MAX_NAME_LENGTH];
   MEM_UINT64  NameLength;
   MEM_STATUS  IOStatus;

   } MEM_IOCTL_FETCH_NAME_ID;

typedef struct _MEM_IOCTL_BUFFER_DATA
   {
   MEM_STATUS  IOStatus;
   MEM_UINT64  Flags;
   MEM_UINT64  BufferHandle;
   MEM_UINT64  DataIndex;
   MEM_UINT64  DataInSize;
   MEM_UINT64  DataOutSize;
   MEM_UINT64  Data;    // must be the last one
   } MEM_IOCTL_BUFFER_DATA;

// function prototypes
// they will all have a C Linkage

#ifdef __cplusplus
extern "C"
{
#endif

MEM_STATUS MEMAttach(PMEM_HANDLE opMEMHandle);
MEM_STATUS MEMDetach(MEM_HANDLE iMEMHandle);
MEM_STATUS MEMAllocBuffer(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferSizeByte,
                          MEM_UINT64 iMemoryBank, MEM_UINT64 iFlags,
                          MEM_UINT64 iProcessId, MEM_UINT64 iOwnerTag,
                          MEM_UINT64 iNameTag, MEM_UINT64 *opBufferHandle,
                          void **opBufferVirtAddr, MEM_UINT64 *opBufferPhysAddr);
MEM_STATUS MEMCreateBuffer(MEM_HANDLE iMEMHandle, MEM_UINT64 iPhysAddr,
                           MEM_UINT64 iBufferSizeByte, MEM_UINT64 iFlags,
                           MEM_UINT64 iProcessId, MEM_UINT64 iOwnerTag,
                           MEM_UINT64 iNameTag, MEM_UINT64 *opBufferHandle, void **opBufferVirtAddr);
MEM_STATUS MEMAllocNamedBuffer(MEM_HANDLE iMEMHandle, const MEM_CHAR* ipName, MEM_UINT64 iBufferSizeByte,
                               MEM_UINT64 iMemoryBank, MEM_UINT64 iFlags, MEM_UINT64 iProcessId, MEM_UINT64 iOwnerTag,
                               MEM_UINT64 iNameTag, MEM_UINT64 *opBufferHandle, void **opBufferVirtAddr, MEM_UINT64 *opBufferPhysAddr);
MEM_STATUS MEMFreeBuffer(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 Reserved1, MEM_UINT64 Reserved2);
MEM_STATUS MEMBufferCleanup(MEM_HANDLE iMEMHandle, MEM_UINT64 iProcessId, MEM_UINT64 iOwnerTag);
MEM_STATUS MEMMapPtr(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, void **opVirtPtr);
MEM_STATUS MEMUnmapPtr(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle);
MEM_STATUS MEMGetBufferInfo(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_BUFFER_INFO* iopBufferInfo);
MEM_STATUS MEMGetBufferInfoAtPosition(MEM_HANDLE iMEMHandle, MEM_UINT64 *iopPosition, MEM_BUFFER_INFO* iopBufferInfo);
MEM_STATUS MEMBufferControl(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 iControlType, MEM_UINT64 iValue);
MEM_STATUS MEMBufferInquire(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 iInquireType, MEM_UINT64 *opValue);
MEM_STATUS MEMGetPhysicalAddress(MEM_HANDLE iMEMHandle, void *iVirtPtr, MEM_UINT64 *opPhysAddr);
MEM_STATUS MEMGetDriverInfo(MEM_HANDLE iMEMHandle, MEM_DRIVER_INFORMATION *iopInfo, MEM_UINT64 iMemBank);
MEM_STATUS MEMAllocUserHeap(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 *opBankHandle);
MEM_STATUS MEMFreeUserHeap(MEM_HANDLE iMEMHandle, MEM_UINT64 *iopBankHandle);
MEM_STATUS MEMOpenNamedBuffer(MEM_HANDLE iMEMHandle, const MEM_CHAR * ipName, MEM_UINT64 iFlags, MEM_UINT64 *opBufferHandle, 
                              void **opBufferVirtAddr, MEM_UINT64 *opBufferPhysAddr, MEM_UINT64 *opBufferSize);
MEM_STATUS MEMBufferRegisterName(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 iFlags, const MEM_CHAR * ipName);
MEM_STATUS MEMBufferFetchNameId(MEM_HANDLE iMEMHandle, MEM_UINT64 iFlags, const MEM_CHAR* ipName, MEM_UINT64 *opBufferHandle);
MEM_STATUS MEMBufferFetchData(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 iIndex, void* opData, MEM_SIZE iDataLength, MEM_SIZE* opFetchLength);
MEM_STATUS MEMBufferStoreData(MEM_HANDLE iMEMHandle, MEM_UINT64 iBufferHandle, MEM_UINT64 iIndex, void* ipData, MEM_SIZE iDataLength);

MEM_UINT64 MemConvertNameTagToValue(char *ipNameTag, MEM_UINT64 iStrLength);
void MemConvertValueToNameTag(MEM_UINT64 iTagValue, char *opNameTag, MEM_UINT64 iArrayLength);

#if MEM_API_DRIVER_CODE && MEM_API_USE_LINUX
int MtxMemKernelIoctl (int cmd, void *msg);
#endif

#ifdef __cplusplus
}
#endif

#endif // __MTXMEMMANAGER_API_H__
